local mod = UpgradeList:NewModule("UI")

local UPGRADELIST_VERSION = GetAddOnMetadata('UpgradeList', 'Version')

function UpgradeList:DrawUI()
	--Create Scrolling Window
	UpgradeListDBVar.profile.messages.scrollwindow = CreateFrame("ScrollingMessageFrame", "Scrolling Window", UIParent, "ChatFrameTemplate");
	UpgradeListDBVar.profile.messages.scrollwindow:SetPoint("CENTER", 0, 200);
	--[[
	UpgradeListDBVar.profile.messages.scrollwindow:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});
	--]]
	UpgradeListDBVar.profile.messages.scrollwindow:SetWidth(UIParent:GetWidth() - 200);
	UpgradeListDBVar.profile.messages.scrollwindow:SetHeight(50);
	UpgradeListDBVar.profile.messages.scrollwindow:SetFadeDuration(2);
	UpgradeListDBVar.profile.messages.scrollwindow:SetFading(true);
	UpgradeListDBVar.profile.messages.scrollwindow:SetHyperlinksEnabled(true);
	UpgradeListDBVar.profile.messages.scrollwindow:SetInsertMode("top");
	UpgradeListDBVar.profile.messages.scrollwindow:SetMaxLines(5);
	UpgradeListDBVar.profile.messages.scrollwindow:SetTimeVisible(10);
	UpgradeListDBVar.profile.messages.scrollwindow:SetJustifyH("CENTER");
	UpgradeListDBVar.profile.messages.scrollwindow:SetJustifyV("CENTER");
	UpgradeListDBVar.profile.messages.scrollwindow:SetFontObject(GameFontNormalHuge);
	UpgradeListDBVar.profile.messages.scrollwindow:SetTextColor(1, 1, 1); --white
	UpgradeListDBVar.profile.messages.scrollwindow:EnableMouse(false);
	UpgradeListDBVar.profile.messages.scrollwindow:Show();
	

	--Create Main Frame (UpgradeListMainFrame)
	local UpgradeListMainFrame = CreateFrame("Frame","UpgradeListMainFrame",UIParent);
	UpgradeListMainFrame:SetFrameStrata("DIALOG");
	UpgradeListMainFrame:SetWidth(1257);
	UpgradeListMainFrame:SetHeight(280);
	UpgradeListMainFrame:SetScale(UpgradeListDBVar.profile.mainframe.scale);
	
	UpgradeListMainFrame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	UpgradeListMainFrame:SetBackdropColor(0,0,0,1);

	UpgradeListMainFrame:SetPoint("TOP",0,-80);
	
	--Makes the Main Frame (and therefore all frames attached to it) draggable
	if UpgradeListDBVar.profile.mainframe.lock == false then
		UpgradeListMainFrame:SetMovable(true)
		UpgradeListMainFrame:EnableMouse(true)
	end
	UpgradeListMainFrame:RegisterForDrag("LeftButton")
	UpgradeListMainFrame:SetScript("OnDragStart", UpgradeListMainFrame.StartMoving)
	UpgradeListMainFrame:SetScript("OnDragStop", UpgradeListMainFrame.StopMovingOrSizing)
	
	tinsert(UISpecialFrames,"UpgradeListMainFrame");


	--Make a title section for the Main Frame 
	local MF_titleBG = UpgradeListMainFrame:CreateTexture(nil,"ARTWORK");
	MF_titleBG:SetTexture("Interface/DialogFrame/UI-DialogBox-Header");
	MF_titleBG:SetWidth(280);
	MF_titleBG:SetHeight(64);
	MF_titleBG:SetPoint("TOP", UpgradeListMainFrame, 0, 12);
	UpgradeListMainFrame.texture = MF_titleBG;

	local MF_titleText = UpgradeListMainFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	MF_titleText:SetText('Upgrade List');
	MF_titleText:SetPoint("TOP", UpgradeListMainFrame, 0, -3);
	
	--Version Identification
	VersionHeading = CreateFrame('Frame', "VersionHeading", UpgradeListMainFrame);
	VersionHeadingFont = VersionHeading:CreateFontString("VersionHeading", "ARTWORK", "GameFontWhiteTiny");
	VersionHeadingFont:SetText("v " .. UPGRADELIST_VERSION);
	VersionHeadingFont:SetPoint("TOPLEFT", UpgradeListMainFrame, "TOPLEFT", 13, -11);
	VersionHeadingFont:SetWidth(VersionHeadingFont:GetStringWidth() + 5);
	VersionHeadingFont:SetHeight(VersionHeadingFont:GetStringHeight());
	
	--Make an invisible frame within the Main Frame that will contain the UpgradeListDBVar.char.SavedOptions.tableUL
	local TableContainer = CreateFrame("Frame", "TableContainer", UpgradeListMainFrame);
	TableContainer:SetWidth(1265);
	TableContainer:SetHeight(235);
	TableContainer:SetPoint("TOPLEFT", UpgradeListMainFrame, "TOPLEFT", 0, -30);
	TableContainer:SetPoint("BOTTOMRIGHT", UpgradeListMainFrame, "BOTTOMRIGHT", 0, -12);

	
	--Create the UpgradeListDBVar.char.SavedOptions.tableUL by defining the columns, rows, height of the rows, then creating the UpgradeListDBVar.char.SavedOptions.tableUL
	local cols = {
		{  -- col 1
			name	= "Priority",
			width	= 42,
			defaultsort = "asc",
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 2
			name	= "iLvl",
			width	= 35,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 3
			name	= "Item",
			width	= 240,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 4
			name	= "Difficulty",
			width	= 80,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 5
			name	= "Instance/Reputation/Point Type/Token",
			width	= 250,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 6
			name	= "Boss/Reputation Level/Points Needed/Token Location",
			width	= 280,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 7
			name	= "Note",
			width	= 280,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
	};

	local rows = 14
	local height = 15
	local UpgradeListScrollingTable = LibStub("ScrollingTable");
	UpgradeListDBVar.char.SavedOptions.tableUL = UpgradeListScrollingTable:CreateST(cols, rows, height, nil, TableContainer); --UpgradeListDBVar.char.SavedOptions.tableUL
	UpgradeListDBVar.char.SavedOptions.tableUL:RegisterEvents({
		["OnClick"] = function (rowFrame, cellFrame, data, cols, row, realrow, column, UpgradeListScrollingTable, ...)
			ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
			if row ~= nil then --if the user isn't clicking in the column header area
				if IsModifiedClick("CTRL") then
					EditNote:Show();
					UpgradeListMainFrame:EnableMouse(false)
					CloseButton:EnableMouse(false)
					ToggleButton:EnableMouse(false)
					UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(false)
					UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(false)
					OptionsFrame:EnableMouse(false)
					ItemEntryBox:EnableMouse(false)
					DropDown_Button:EnableMouse(false)
					NoteEntryBox:EnableMouse(false)
					PrimarySpecRadioButton:EnableMouse(false)
					SecondarySpecRadioButton:EnableMouse(false)
					priorityfield:EnableMouse(false)
					AddButton:EnableMouse(false)
					tofield:EnableMouse(false)
					fromfield:EnableMouse(false)
					MoveButton:EnableMouse(false)
					DeleteButton:EnableMouse(false)
					deletefield:EnableMouse(false)
				end
				if UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() then --If the primary checkbox is checked
					if PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 1 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Head.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Head.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Head.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 2 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Neck.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Neck.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Neck.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 3 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Shoulder.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Shoulder.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Shoulder.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 4 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Back.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Back.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Back.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 5 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Chest.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Chest.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Chest.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 6 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Wrist.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Wrist.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Wrist.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 7 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Hand.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Hand.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Hand.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 8 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Waist.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Waist.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Waist.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 9 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Leg.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Leg.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Leg.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 10 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Foot.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Foot.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Foot.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 11 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Ring.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Ring.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Ring.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 12 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Trinket.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Trinket.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Trinket.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Main.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Main.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Main.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Off.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Off.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Off.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 15 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Ranged.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Ranged.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Ranged.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 16 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.PrimarySpecList.Other.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.PrimarySpecList.Other.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.PrimarySpecList.Other.Row[realrow][3]);
						end
					end
				else --The secondary checkbox is checked
					if PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 1 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Head.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Head.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Head.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 2 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Neck.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Neck.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Neck.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 3 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Shoulder.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Shoulder.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Shoulder.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 4 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Back.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Back.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Back.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 5 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Chest.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Chest.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Chest.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 6 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Wrist.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Wrist.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Wrist.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 7 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Hand.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Hand.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Hand.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 8 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Waist.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Waist.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Waist.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 9 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Leg.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Leg.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Leg.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 10 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Foot.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Foot.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Foot.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 11 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Ring.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Ring.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Ring.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 12 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Trinket.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Trinket.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Trinket.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Main.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Main.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Main.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Off.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Off.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Off.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 15 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Ranged.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Ranged.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Ranged.Row[realrow][3]);
						end
					elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 16 then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char.SecondarySpecList.Other.Row[realrow][7] = newnote;
								UpgradeListMainFrame:EnableMouse(true)
								CloseButton:EnableMouse(true)
								ToggleButton:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
								UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
								OptionsFrame:EnableMouse(true)
								ItemEntryBox:EnableMouse(true)
								DropDown_Button:EnableMouse(true)
								NoteEntryBox:EnableMouse(true)
								PrimarySpecRadioButton:EnableMouse(true)
								SecondarySpecRadioButton:EnableMouse(true)
								priorityfield:EnableMouse(true)
								AddButton:EnableMouse(true)
								tofield:EnableMouse(true)
								fromfield:EnableMouse(true)
								MoveButton:EnableMouse(true)
								DeleteButton:EnableMouse(true)
								deletefield:EnableMouse(true)
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char.SecondarySpecList.Other.Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char.SecondarySpecList.Other.Row[realrow][3]);
						end
					end
				end
			end
			ShowUIPanel(ItemRefTooltip)
		end,
	});
	
	--"Close" button
	local CloseButton = CreateFrame('Button', "CloseButton", UpgradeListMainFrame, "UIPanelCloseButton");
	CloseButton:SetPoint("TOPRIGHT", -1, -2);
	CloseButton:SetWidth(40);
	CloseButton:SetHeight(40);
	
	--"ToggleOptions" button
	local ToggleButton = CreateFrame('Button', "ToggleButton", UpgradeListMainFrame, "UIPanelButtonTemplate2");
	ToggleButton:SetPoint("RIGHT", CloseButton, "LEFT", 8, 0);
	ToggleButton:SetWidth(170);
	ToggleButton:SetHeight(20);
	ToggleButton:SetText("Toggle Options Window");
	ToggleButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:ToggleOptions();
	end);
	
	--"PrimarySpecView" button
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck = CreateFrame('CheckButton', "UpgradeListDBVar.char.SavedOptions.PrimaryCheck", UpgradeListMainFrame, "UICheckButtonTemplate");
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetPoint("RIGHT", ToggleButton, "LEFT", -275, -2);
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(true);
	_G[UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetName() .. "Text"]:SetText("View Primary Spec");
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetScript("OnClick", function (self, button, down)
		if GetNumTalentGroups("player") == 1 then
			UpgradeListDBVar.char.SavedOptions.SecondaryCheck:Disable();
			UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(true);
		else
			UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetChecked(false);
			UpgradeList:RefreshWindow()
		end
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(true);
	end);
	
	--"SecondarySpecView" button
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck = CreateFrame('CheckButton', "UpgradeListDBVar.char.SavedOptions.SecondaryCheck", UpgradeListMainFrame, "UICheckButtonTemplate");
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetPoint("RIGHT", ToggleButton, "LEFT", -125, -2);
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetChecked(false);
	_G[UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetName() .. "Text"]:SetText("View Secondary Spec");
	if GetNumTalentGroups("player") == 1 then
		_G[UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetName() .. "Text"]:SetTextColor(0.5, 0.5, 0.5);
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:Disable();
	end
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetScript("OnClick", function (self, button, down)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetChecked(true);
		UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(false);
		UpgradeList:RefreshWindow()
	end);
	
	--Explanation of what the various clicks on the table do
	ExplanationHeading = CreateFrame('Frame', "ExplanationHeading", UpgradeListMainFrame);
	ExplanationHeadingFont = ExplanationHeading:CreateFontString("ExplanationHeading", "ARTWORK", "GameFontWhiteTiny");
	ExplanationHeadingFont:SetText("Clicking a row will show the tooltip for the item, Shift-clicking a row will allow you to add the item link into your chat window, Ctrl-clicking a row will allow you to edit the note for that row.");
	ExplanationHeadingFont:SetPoint("TOPLEFT", UpgradeListMainFrame, "TOPLEFT", 50, -11);
	ExplanationHeadingFont:SetWidth(500);
	ExplanationHeadingFont:SetHeight(ExplanationHeadingFont:GetStringHeight());
	ExplanationHeadingFont:SetJustifyH("LEFT");

	--Create all the 16 tabs for the equipment slots
	PanelTemplates_SetNumTabs(UpgradeListMainFrame, 16);
	Tab_1 = CreateFrame('Button', "$parentTab1", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_1:SetID(1);
	Tab_1:SetText('Head');
	Tab_1:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -575, 6);
	PanelTemplates_TabResize(Tab_1, 0, 93, 94);
	Tab_1:RegisterForClicks("AnyUp");
	Tab_1:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_1, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);

	Tab_2 = CreateFrame('Button', "$parentTab2", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_2:SetID(2);
	Tab_2:SetText('Neck');
	Tab_2:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -499, 6);
	PanelTemplates_TabResize(Tab_2, 0, 93, 94);
	Tab_2:RegisterForClicks("AnyUp");
	Tab_2:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_2, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_3 = CreateFrame('Button', "$parentTab3", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_3:SetID(3);
	Tab_3:SetText('Shoulders');
	Tab_3:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -423, 6);
	PanelTemplates_TabResize(Tab_3, 0, 93, 94);
	Tab_3:RegisterForClicks("AnyUp");
	Tab_3:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_3, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_4 = CreateFrame('Button', "$parentTab4", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_4:SetID(4);
	Tab_4:SetText('Back');
	Tab_4:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -347, 6);
	PanelTemplates_TabResize(Tab_4, 0, 93, 94);
	Tab_4:RegisterForClicks("AnyUp");
	Tab_4:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_4, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_5 = CreateFrame('Button', "$parentTab5", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_5:SetID(5);
	Tab_5:SetText('Chest');
	Tab_5:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -271, 6);
	PanelTemplates_TabResize(Tab_5, 0, 93, 94);
	Tab_5:RegisterForClicks("AnyUp");
	Tab_5:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_5, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_6 = CreateFrame('Button', "$parentTab6", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_6:SetID(6);
	Tab_6:SetText('Wrists');
	Tab_6:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -195, 6);
	PanelTemplates_TabResize(Tab_6, 0, 93, 94);
	Tab_6:RegisterForClicks("AnyUp");
	Tab_6:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_6, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_7 = CreateFrame('Button', "$parentTab7", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_7:SetID(7);
	Tab_7:SetText('Hands');
	Tab_7:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -119, 6);
	PanelTemplates_TabResize(Tab_7, 0, 93, 94);
	Tab_7:RegisterForClicks("AnyUp");
	Tab_7:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_7, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_8 = CreateFrame('Button', "$parentTab8", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_8:SetID(8);
	Tab_8:SetText('Waist');
	Tab_8:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -43, 6);
	PanelTemplates_TabResize(Tab_8, 0, 93, 94);
	Tab_8:RegisterForClicks("AnyUp");
	Tab_8:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_8, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_9 = CreateFrame('Button', "$parentTab9", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_9:SetID(9);
	Tab_9:SetText('Legs');
	Tab_9:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 33, 6);
	PanelTemplates_TabResize(Tab_9, 0, 93, 94);
	Tab_9:RegisterForClicks("AnyUp");
	Tab_9:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_9, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_10 = CreateFrame('Button', "$parentTab10", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_10:SetID(10);
	Tab_10:SetText('Feet');
	Tab_10:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 109, 6);
	PanelTemplates_TabResize(Tab_10, 0, 93, 94);
	Tab_10:RegisterForClicks("AnyUp");
	Tab_10:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_10, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_11 = CreateFrame('Button', "$parentTab11", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_11:SetID(11);
	Tab_11:SetText('Rings');
	Tab_11:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 185, 6);
	PanelTemplates_TabResize(Tab_11, 0, 93, 94);
	Tab_11:RegisterForClicks("AnyUp");
	Tab_11:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_11, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_12 = CreateFrame('Button', "$parentTab12", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_12:SetID(12);
	Tab_12:SetText('Trinkets');
	Tab_12:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 261, 6);
	PanelTemplates_TabResize(Tab_12, 0, 93, 94);
	Tab_12:RegisterForClicks("AnyUp");
	Tab_12:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_12, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_13 = CreateFrame('Button', "$parentTab13", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_13:SetID(13);
	Tab_13:SetText('Main Hand');
	Tab_13:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 342, 6);
	PanelTemplates_TabResize(Tab_13, 0, 102, 90);
	Tab_13:RegisterForClicks("AnyUp");
	Tab_13:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_13, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_14 = CreateFrame('Button', "$parentTab14", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_14:SetID(14);
	Tab_14:SetText('Off Hand');
	Tab_14:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 423, 6);
	PanelTemplates_TabResize(Tab_14, 0, 94, 94);
	Tab_14:RegisterForClicks("AnyUp");
	Tab_14:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_14, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_15 = CreateFrame('Button', "$parentTab15", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_15:SetID(15);
	Tab_15:SetText('Ranged');
	Tab_15:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 499, 6);
	PanelTemplates_TabResize(Tab_15, 0, 94, 94);
	Tab_15:RegisterForClicks("AnyUp");
	Tab_15:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_15, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_16 = CreateFrame('Button', "$parentTab16", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_16:SetID(16);
	Tab_16:SetText('Other');
	Tab_16:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 575, 6);
	PanelTemplates_TabResize(Tab_16, 0, 94, 94);
	Tab_16:RegisterForClicks("AnyUp");
	Tab_16:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_16, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	PanelTemplates_SetTab(UpgradeListMainFrame, 1);
	UpgradeList:RefreshWindow()
	
	--Create the options frame below the main frame
	OptionsFrame = CreateFrame("Frame", "OptionsFrame", UpgradeListMainFrame);
	OptionsFrame:SetWidth(850);
	OptionsFrame:SetHeight(330);
	OptionsFrame:SetPoint("TOP",UpgradeListMainFrame,"BOTTOM", 0, -30);
	OptionsFrame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 12, top = 12, bottom = 11 }});
	
	--Creation of the Item Entry Box
	ItemEntryBox = CreateFrame('EditBox', "ItemEntryBox", OptionsFrame, "InputBoxTemplate");
	ItemEntryBox:SetPoint("CENTER", 0, 125);
	ItemEntryBox:SetAutoFocus(false);
	ItemEntryBox:SetWidth(130);
    ItemEntryBox:SetHeight(24);
	
	ItemEntryBoxHeading = CreateFrame('Frame', "ItemEntryBoxHeading", OptionsFrame);
	HeadingFont = ItemEntryBoxHeading:CreateFontString("ItemHeading", "ARTWORK", "GameFontNormalSmallLeft");
	HeadingFont:SetText("Enter your item with an item link or an item ID number:");
	HeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP");
	HeadingFont:SetWidth(HeadingFont:GetStringWidth() + 5);
	HeadingFont:SetHeight(HeadingFont:GetStringHeight());	

	--Creation of the dropdown menus
	CreateFrame("Button", "DropDown_Button", OptionsFrame, "UIDropDownMenuTemplate")
	DropDown_Button:ClearAllPoints()
	DropDown_Button:SetPoint("TOP", ItemEntryBox, "BOTTOM", 0, -20)
	DropDown_Button:Show()
	

	function DropDown_InitializeDropDown(self, level)

		--Top Level
		if level == 1 then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:TopLevel(info, level)

		--Raid Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == "Raid") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:RaidSubmenu(info, level)
			
		--Dungeon Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == "Dungeon") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:DungeonSubmenu(info, level)
			
		--Crafted Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == "Crafted") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:CraftedSubmenu(info, level)
			
		--Quest Reward Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == "Quest Reward") then
			local info = UIDropDownMenu_CreateInfo()
			--local DropLocation = "Quest Reward"
			
			info.text = "Quest Reward"
			info.value = "Quest Reward"
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(DropDown_Button, "Quest Reward");
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = "Quest Reward";
				CloseDropDownMenus(1);
			end
			UIDropDownMenu_AddButton(info, level)

		--Reputation Reward Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == "Reputation Reward") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Reputations(info, level)
		
		--Purchasable Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == "Purchasable") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Purchasable(info, level)
			
		--World Drop Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == "World Drop") then
			local info = UIDropDownMenu_CreateInfo()
			
			info.text = "World Drop"
			info.value = "World Drop"
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(DropDown_Button, "World Drop");
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = "World Drop";
				CloseDropDownMenus(1);
			end
			UIDropDownMenu_AddButton(info, level)

		--List of heroic raids
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == "HeroicRaid") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HeroicRaids(info, level)
			
		--List of normal raids
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == "NormalRaid") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:NormalRaids(info, level)
		
		--Heroic Dungeons
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == "HeroicDungeon") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HeroicDungeons(info, level)
			
		--Normal Dungeons
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == "NormalDungeon") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:NormalDungeons(info, level)
		
		--Reputation Levels
		elseif (level == 3) and ((UIDROPDOWNMENU_MENU_VALUE == "Guardians of Hyjal")
		or (UIDROPDOWNMENU_MENU_VALUE == "The Earthen Ring")
		or (UIDROPDOWNMENU_MENU_VALUE == "Therazane")
		or (UIDROPDOWNMENU_MENU_VALUE == "Ramkahen")
		or (UIDROPDOWNMENU_MENU_VALUE == "Wildhammer Clan")
		or (UIDROPDOWNMENU_MENU_VALUE == "Baradin's Wardens")
		or (UIDROPDOWNMENU_MENU_VALUE == "Dragonmaw Clan")
		or (UIDROPDOWNMENU_MENU_VALUE == "Hellscream's Reach")) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:RepLevels(info, level, UIDROPDOWNMENU_MENU_VALUE)
		
		--Tier Selection for Tokens
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == "Gear Token") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:TierSelection(info, level)
			
		--Tier Tokens
		elseif (level == 4) and ((UIDROPDOWNMENU_MENU_VALUE == "Tier 11")
		or (UIDROPDOWNMENU_MENU_VALUE == "Tier 12")) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:GearTokens(info, level, UIDROPDOWNMENU_MENU_VALUE)

		--Normal Baradin Hold Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Baradin Hold Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BaradinHold(info, level, "Normal")
			
		--Heroic Baradin Hold Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Baradin Hold Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BaradinHold(info, level, "Heroic")
			
		--Normal Bastion of Twilight Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Bastion of Twilight Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BastionOfTwilight(info, level, "Normal")
		
		--Heroic Bastion of Twilight Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Bastion of Twilight Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BastionOfTwilight(info, level, "Heroic")
			
		--Normal Blackwing Descent Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Blackwing Descent Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackwingDescent(info, level, "Normal")
			
		--Heroic Blackwing Descent Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Blackwing Descent Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackwingDescent(info, level, "Heroic")
		
		--Normal Throne of the Four Winds Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Throne of the Four Winds Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTheFourWinds(info, level, "Normal")
			
		--Heroic Throne of the Four Winds Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Throne of the Four Winds Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTheFourWinds(info, level, "Heroic")
			
		--Heroic Zul'Aman Bosses
		elseif (level==4) and (UIDROPDOWNMENU_MENU_VALUE == "Zul'Aman Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ZulAman(info, level)
			
		--Heroic Zul'Gurub Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Zul'Gurub Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ZulGurub(info, level)
			
		--Heroic Throne of Tides Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Throne of the Tides Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTides(info, level, "Heroic")
			
		--Heroic Blackrock Caverns Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Blackrock Caverns Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackrockCaverns(info, level, "Heroic")
			
		--Heroic Stonecore Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "The Stonecore Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Stonecore(info, level, "Heroic")

		--Heroic Vortex Pinnacle Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "The Vortex Pinnacle Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:VortexPinnacle(info, level, "Heroic")

		--Heroic Lost City of Tol'vir Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Lost City of Tol'vir Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:LostCityOfTolvir(info, level, "Heroic")
			
		--Heroic Grim Batol Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Grim Batol Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:GrimBatol(info, level, "Heroic")
			
		--Heroic Halls of Origination Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Halls of Origination Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HallsOfOrigination(info, level, "Heroic")
			
		--Heroic Deadmines Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "The Deadmines Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Deadmines(info, level)
			
		--Heroic Shadowfang Keep Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Shadowfang Keep Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ShadowfangKeep(info, level)
			
		--Normal Throne of Tides Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Throne of the Tides Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTides(info, level, "Normal")
			
		--Normal Blackrock Caverns Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Blackrock Caverns Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackrockCaverns(info, level, "Normal")
		
		--Normal Stonecore Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "The Stonecore Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Stonecore(info, level, "Normal")
		
		--Normal Vortex Pinnacle Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "The Vortex Pinnacle Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:VortexPinnacle(info, level, "Normal")
		
		--Normal Lost City of Tol'vir Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Lost City of Tol'vir Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:LostCityOfTolvir(info, level, "Normal")
		
		--Normal Grim Batol Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Grim Batol Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:GrimBatol(info, level, "Normal")
		
		--Normal Halls of Origination Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "Halls of Origination Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HallsOfOrigination(info, level, "Normal")
		
		end
	end

	
	function DropDown_DropDownOnLoad(self)
		UIDropDownMenu_Initialize(self, DropDown_InitializeDropDown)
	end

	function DropDown_ButtonOnClick(self, button, down)
		local name = self:GetName()
		ToggleDropDownMenu(1, nil, DropDown_DropDown, name, 0, 0)
	end
	
	UIDropDownMenu_Initialize(DropDown_Button, DropDown_InitializeDropDown)
	UIDropDownMenu_SetWidth(DropDown_Button, 175);
	UIDropDownMenu_SetButtonWidth(DropDown_Button, 175)
	UIDropDownMenu_SetSelectedID(DropDown_Button, 0)
	UIDropDownMenu_JustifyText(DropDown_Button, "CENTER")
	UIDropDownMenu_SetText(DropDown_Button, "Location:")

	local DropDownHeading = CreateFrame('Frame', "DropDownHeading", OptionsFrame);
	NoteFont = DropDownHeading:CreateFontString("DropDownHeading", "ARTWORK", "GameFontNormalSmallLeft");
	NoteFont:SetText("Select Item Location:");
	NoteFont:SetPoint("BOTTOM", DropDown_Button, "TOP");
	NoteFont:SetWidth(NoteFont:GetStringWidth() + 5);
	NoteFont:SetHeight(NoteFont:GetStringHeight());	
	
	--Creation of the Note Entry Box
	local NoteEntryBox = CreateFrame('EditBox', "NoteEntryBox", OptionsFrame, "InputBoxTemplate");
	NoteEntryBox:SetPoint("TOP", DropDown_Button, "BOTTOM", 0, -20);
	NoteEntryBox:SetAutoFocus(false);
	NoteEntryBox:SetWidth(250);
    NoteEntryBox:SetHeight(24);
	NoteEntryBox:SetMaxLetters(28);
	
	local NoteEntryBoxHeading = CreateFrame('Frame', "NoteEntryBoxHeading", OptionsFrame);
	NoteFont = NoteEntryBoxHeading:CreateFontString("NoteHeading", "ARTWORK", "GameFontNormalSmallLeft");
	NoteFont:SetText("Enter an optional note:");
	NoteFont:SetPoint("BOTTOM", NoteEntryBox, "TOP");
	NoteFont:SetWidth(NoteFont:GetStringWidth() + 5);
	NoteFont:SetHeight(NoteFont:GetStringHeight());	
	
	--Primary Spec Selector Radio Button
	local PrimarySpecRadioButton = CreateFrame('CheckButton', "PrimarySpecRadioButton", OptionsFrame, "UIRadioButtonTemplate");
	PrimarySpecRadioButton:SetHeight(20);
	PrimarySpecRadioButton:SetWidth(20);
	PrimarySpecRadioButton:SetChecked(true);
	PrimarySpecRadioButton:SetPoint("TOP", NoteEntryBox, "BOTTOM", -95, -25);
	_G[PrimarySpecRadioButton:GetName() .. "Text"]:SetText("Primary Spec");
	PrimarySpecRadioButton:SetScript("OnClick", function (self, button, down)
		if GetNumTalentGroups("player") == 1 then
			SecondarySpecRadioButton:Disable();
			PrimarySpecRadioButton:SetChecked(true);
		else
			SecondarySpecRadioButton:SetChecked(false);
		end
	PrimarySpecRadioButton:SetChecked(true);
	end);
	
	--Secondary Spec Selector Radio Button
	local SecondarySpecRadioButton = CreateFrame('CheckButton', "SecondarySpecRadioButton", OptionsFrame, "UIRadioButtonTemplate");
	SecondarySpecRadioButton:SetHeight(20);
	SecondarySpecRadioButton:SetWidth(20);
	SecondarySpecRadioButton:SetChecked(false);
	SecondarySpecRadioButton:SetPoint("TOP", NoteEntryBox, "BOTTOM", 20, -25);
	_G[SecondarySpecRadioButton:GetName() .. "Text"]:SetText("Secondary Spec");
	if GetNumTalentGroups("player") == 1 then
		_G[SecondarySpecRadioButton:GetName() .. "Text"]:SetTextColor(0.5, 0.5, 0.5);
		SecondarySpecRadioButton:Disable();
	end
	SecondarySpecRadioButton:SetScript("OnClick", function (self, button, down)
		SecondarySpecRadioButton:SetChecked(true);
		PrimarySpecRadioButton:SetChecked(false);
	end);
	
	local SpecEntryBoxHeading = CreateFrame('Frame', "SpecEntryBoxHeading", OptionsFrame);
	SpecFont = SpecEntryBoxHeading:CreateFontString("SpecHeading", "ARTWORK", "GameFontNormalSmallLeft");
	SpecFont:SetText("Select which spec to which you would like to add this item:");
	SpecFont:SetPoint("BOTTOM", NoteEntryBox, "TOP", 0, -47);
	SpecFont:SetWidth(SpecFont:GetStringWidth() + 5);
	SpecFont:SetHeight(SpecFont:GetStringHeight());	
	
	--AddItemLocation
	local priorityfield = CreateFrame('EditBox', "priorityfield", OptionsFrame, "InputBoxTemplate");
	priorityfield:SetPoint("TOP", ItemEntryBox, "BOTTOM", 0, -180);
	priorityfield:SetAutoFocus(false);
	priorityfield:SetWidth(25);
	priorityfield:SetHeight(24);
	
	local PriorityBoxHeading = CreateFrame('Frame', "PriorityBoxHeading", OptionsFrame);
	PriorityFont = PriorityBoxHeading:CreateFontString("SpecHeading", "ARTWORK", "GameFontNormalSmallLeft");
	PriorityFont:SetText("Enter the priority number for this item OR \nleave it blank to add the item to bottom of the list:");
	PriorityFont:SetJustifyH("CENTER");
	PriorityFont:SetPoint("BOTTOM", priorityfield, "TOP", 0, 5);
	PriorityFont:SetWidth(PriorityFont:GetStringWidth() + 5);
	PriorityFont:SetHeight(PriorityFont:GetStringHeight());	
	
	
	--"Add" button
	local AddButton = CreateFrame('Button', "AddButton", OptionsFrame, "UIPanelButtonTemplate");
	AddButton:SetPoint("TOP", priorityfield, "BOTTOM", 0, -5);
	AddButton:SetWidth(100);
    AddButton:SetHeight(24);
	AddButton:SetText("Add Item");
	AddButton:SetScript("OnEnter", function (self, motion)
		if ItemEntryBox ~= "" then
			UpgradeList:LoadItemIntoCache(ItemEntryBox);
		end
	end);
	AddButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:AddItem(ItemEntryBox, PrimarySpecRadioButton, SecondarySpecRadioButton, UpgradeListMainFrame, priorityfield, statusbarfont, NoteEntryBox);
	end);
	
	--Moving heading text
	MovingHeading = CreateFrame('Frame', "MovingHeading", OptionsFrame);
	MovingHeadingFont = MovingHeading:CreateFontString("MovingHeading", "ARTWORK", "GameFontNormalSmallLeft");
	MovingHeadingFont:SetText("In the currently selected tab, move item #:");
	MovingHeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP", -275, 0);
	MovingHeadingFont:SetWidth(MovingHeadingFont:GetStringWidth() + 5);
	MovingHeadingFont:SetHeight(MovingHeadingFont:GetStringHeight());
	
	--"From" field for moving items
	local fromfield = CreateFrame('EditBox', "fromfield", OptionsFrame, "InputBoxTemplate");
	fromfield:SetPoint("TOP", MovingHeadingFont, "BOTTOM", -30, 0);
	fromfield:SetAutoFocus(false);
	fromfield:SetWidth(25);
	fromfield:SetHeight(24);
	
	--"To" field for moving items
	local tofield = CreateFrame('EditBox', "tofield", OptionsFrame, "InputBoxTemplate");
	tofield:SetPoint("LEFT", fromfield, "RIGHT", 20, 0);
	tofield:SetAutoFocus(false);
	tofield:SetWidth(25);
	tofield:SetHeight(24);
	
	--"To" between the To and From fields
	local to = CreateFrame('Frame', "To", OptionsFrame);
	toheadingfont = to:CreateFontString("to", "ARTWORK", "GameFontNormalSmallLeft");
	toheadingfont:SetText("to");
	toheadingfont:SetPoint("LEFT", fromfield, "RIGHT", 2, 0);
	
	--"Move Item" button
	local MoveButton = CreateFrame('Button', "MoveButton", OptionsFrame, "UIPanelButtonTemplate");
	MoveButton:SetPoint("TOP", toheadingfont, "BOTTOM", 0, -10);
	MoveButton:SetWidth(100);
    MoveButton:SetHeight(24);
	MoveButton:SetText("Move Item");
	MoveButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:MoveItem(fromfield, tofield, UpgradeListMainFrame);
	end);
	
	--"Delete item" heading
	DeletingHeading = CreateFrame('Frame', "DeletingHeading", OptionsFrame);
	DeletingHeadingFont = DeletingHeading:CreateFontString("DeletingHeading", "ARTWORK", "GameFontNormalSmallLeft");
	DeletingHeadingFont:SetText("In the currently selected tab, delete item #:");
	DeletingHeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP", 275, 0);
	DeletingHeadingFont:SetWidth(DeletingHeadingFont:GetStringWidth() + 5);
	DeletingHeadingFont:SetHeight(DeletingHeadingFont:GetStringHeight());
	
	--"Delete Item" button
	local DeleteButton = CreateFrame('Button', "DeleteButton", OptionsFrame, "UIPanelButtonTemplate");
	DeleteButton:SetPoint("TOP", DeletingHeadingFont, "BOTTOM", 0, -25);
	DeleteButton:SetWidth(100);
    DeleteButton:SetHeight(24);
	DeleteButton:SetText("Delete Item");
	DeleteButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:DeleteItem(deletefield, UpgradeListMainFrame, statusbarfont);
	end);
	
	--"Delete Item" field
	local deletefield = CreateFrame('EditBox', "deletefield", OptionsFrame, "InputBoxTemplate");
	deletefield:SetPoint("TOP", DeletingHeadingFont, "BOTTOM", 0, 0);
	deletefield:SetAutoFocus(false);
	deletefield:SetWidth(25);
	deletefield:SetHeight(24);
	
	--Status Bar
	local PaneBackdrop  = {
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 5, bottom = 3 }
	}
	
	local statusbar = CreateFrame("Frame", "Status Bar", OptionsFrame)
	statusbar:SetPoint("BOTTOMLEFT", OptionsFrame, "BOTTOMLEFT", 15, 15)
	statusbar:SetPoint("BOTTOMRIGHT", OptionsFrame, "BOTTOMRIGHT", -15, 15)
	statusbar:SetHeight(24)
	statusbar:SetBackdrop(PaneBackdrop)
	statusbar:SetBackdropColor(0.1,0.1,0.1)
	statusbar:SetBackdropBorderColor(0.4,0.4,0.4)
	
	statusbarfont = statusbar:CreateFontString("statusbar", "ARTWORK", "GameFontWhite");
	statusbarfont:SetPoint("LEFT", statusbar, "LEFT", 7, 0);
	statusbarfont:SetHeight(22);
	statusbarfont:SetText("UpgradeList v " .. UPGRADELIST_VERSION);
	
	--Frame for entering the number of points needed for certain items
	local PointInput = CreateFrame("Frame", "PointInput", UIParent);
	PointInput:SetFrameStrata("FULLSCREEN");
	PointInput:SetWidth(250);
	PointInput:SetHeight(200);
	
	PointInput:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
	edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
	tile = true, tileSize = 32, edgeSize = 32, 
	insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	PointInput:SetBackdropColor(0,0,0,1);
	PointInput:SetPoint("CENTER", UIParent, "CENTER");
	
	PointInput:Hide();
	
	local PointInputHeading = CreateFrame('Frame', "PointInputHeading", PointInput);
	PointInputFont = PointInputHeading:CreateFontString("PointInputHeading", "ARTWORK", "GameFontNormal");
	PointInputFont:SetText("Enter the number of	points\n needed for this item:");
	PointInputFont:SetJustifyH("CENTER");
	PointInputFont:SetPoint("TOP", PointInput, "TOP", 0, -15);
	PointInputFont:SetWidth(230);
	PointInputFont:SetHeight(PointInputFont:GetStringHeight());
	
	local PointInputBox = CreateFrame("Editbox", "PointInputBox", PointInput, "InputBoxTemplate");
	PointInputBox:SetPoint("CENTER", PointInput, "CENTER", 0, 0);
	PointInputBox:SetAutoFocus(true);
	PointInputBox:SetWidth(175);
	PointInputBox:SetHeight(27);
	PointInputBox:SetNumeric(1);
	PointInputBox:SetMaxLetters(5);
	
	local PointInputOK = CreateFrame('Button', "PointInputOK", PointInput, "UIPanelButtonTemplate");
	PointInputOK:SetPoint("BOTTOM", PointInput, "BOTTOM", -50, 20);
	PointInputOK:SetWidth(100);
	PointInputOK:SetHeight(24);
	PointInputOK:SetText("Ok");
	PointInputOK:SetScript("OnClick", function (self, button, down)
		local pointnum = tonumber(PointInputBox:GetText());
		if pointnum == nil then
			statusbarfont:SetTextColor(1, 0, 0);
			statusbarfont:SetText("No number entered.");
		elseif pointnum >= 0 then
			pointnum = math.floor(pointnum);
			UIDropDownMenu_SetText(DropDown_Button, UIDropDownMenu_GetSelectedName(DropDown_Button));
			PointInput:Hide();
			PointInputBox:SetText("");
			UpgradeListDBVar.char.SavedOptions.SelectedBoss = pointnum;
			UpgradeListDBVar.char.SavedOptions.SelectedInstance = UIDropDownMenu_GetSelectedName(DropDown_Button);
			UpgradeListMainFrame:EnableMouse(true)
			CloseButton:EnableMouse(true)
			ToggleButton:EnableMouse(true)
			UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
			UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
			OptionsFrame:EnableMouse(true)
			ItemEntryBox:EnableMouse(true)
			DropDown_Button:EnableMouse(true)
			NoteEntryBox:EnableMouse(true)
			PrimarySpecRadioButton:EnableMouse(true)
			SecondarySpecRadioButton:EnableMouse(true)
			priorityfield:EnableMouse(true)
			AddButton:EnableMouse(true)
			tofield:EnableMouse(true)
			fromfield:EnableMouse(true)
			MoveButton:EnableMouse(true)
			DeleteButton:EnableMouse(true)
			deletefield:EnableMouse(true)
		else
			statusbarfont:SetTextColor(1, 0, 0);
			statusbarfont:SetText("Invalid number.");
		end
	end);
	
	local PointInputCancel = CreateFrame('Button', "PointInputCancel", PointInput, "UIPanelButtonTemplate");
	PointInputCancel:SetPoint("LEFT", PointInputOK, "RIGHT", 0, 0);
	PointInputCancel:SetWidth(100);
	PointInputCancel:SetHeight(24);
	PointInputCancel:SetText("Cancel");
	PointInputCancel:SetScript("OnClick", function (self, button, down)
		UIDropDownMenu_SetText(DropDown_Button, "Location:");
		PointInput:Hide();
		UpgradeListMainFrame:EnableMouse(true)
		CloseButton:EnableMouse(true)
		ToggleButton:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
		OptionsFrame:EnableMouse(true)
		ItemEntryBox:EnableMouse(true)
		DropDown_Button:EnableMouse(true)
		NoteEntryBox:EnableMouse(true)
		PrimarySpecRadioButton:EnableMouse(true)
		SecondarySpecRadioButton:EnableMouse(true)
		priorityfield:EnableMouse(true)
		AddButton:EnableMouse(true)
		tofield:EnableMouse(true)
		fromfield:EnableMouse(true)
		MoveButton:EnableMouse(true)
		DeleteButton:EnableMouse(true)
		deletefield:EnableMouse(true)
		PointInputBox:SetText("");
	end);
	
	--Frame for selecting whether a weapon is main hand or off hand.
	local WeaponInput = CreateFrame("Frame", "WeaponInput", UIParent);
	WeaponInput:SetFrameStrata("FULLSCREEN");
	WeaponInput:SetWidth(250);
	WeaponInput:SetHeight(200);
	
	WeaponInput:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	WeaponInput:SetBackdropColor(0,0,0,1);
	WeaponInput:SetPoint("CENTER", UIParent, "CENTER");
	
	WeaponInput:Hide();
	
	local WeaponInputHeading = CreateFrame('Frame', "WeaponInputHeading", WeaponInput);
	WeaponInputFont = WeaponInputHeading:CreateFontString("WeaponInputHeading", "ARTWORK", "GameFontNormal");
	WeaponInputFont:SetText("Select which list to add this item:");
	WeaponInputFont:SetJustifyH("CENTER");
	WeaponInputFont:SetPoint("TOP", WeaponInput, "TOP", 0, -15);
	WeaponInputFont:SetWidth(230);
	WeaponInputFont:SetHeight(WeaponInputFont:GetStringHeight());
	
	--Main Hand weapon checkbox
	local MainHandCheck = CreateFrame('CheckButton', "MainHandCheck", WeaponInput, "UICheckButtonTemplate");
	MainHandCheck:SetPoint("CENTER", WeaponInput, "CENTER", -30, 25);
	MainHandCheck:SetChecked(true);
	_G[MainHandCheck:GetName() .. "Text"]:SetText("Main Hand");
	MainHandCheck:SetScript("OnClick", function (self, button, down)
		OffHandCheck:SetChecked(false);
		MainHandCheck:SetChecked(true);
	end);
	
	--Off Hand weapon checkbox
	local OffHandCheck = CreateFrame('CheckButton', "OffHandCheck", WeaponInput, "UICheckButtonTemplate");
	OffHandCheck:SetPoint("TOP", MainHandCheck, "BOTTOM", 0, 0);
	OffHandCheck:SetChecked(false);
	_G[OffHandCheck:GetName() .. "Text"]:SetText("Off Hand");
	OffHandCheck:SetScript("OnClick", function (self, button, down)
		OffHandCheck:SetChecked(true);
		MainHandCheck:SetChecked(false);
	end);
	
	local WeaponInputOK = CreateFrame('Button', "WeaponInputOK", WeaponInput, "UIPanelButtonTemplate");
	WeaponInputOK:SetPoint("BOTTOM", WeaponInput, "BOTTOM", -50, 20);
	WeaponInputOK:SetWidth(100);
	WeaponInputOK:SetHeight(24);
	WeaponInputOK:SetText("Ok");
	WeaponInputOK:SetScript("OnClick", function (self, button, down)
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(ItemEntryBox:GetText());
		local priorityfieldnum = tonumber(priorityfield:GetText());
		if priorityfieldnum ~= nil then
			priorityfieldnum = math.floor(priorityfieldnum);
		end
		if PrimarySpecRadioButton:GetChecked() then --Adding to Primary Spec List
			if MainHandCheck:GetChecked() then --If the user selects the Main Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.PrimarySpecList.Main.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.PrimarySpecList.Main.Row[UpgradeListDBVar.char.PrimarySpecList.Main.Count] = { UpgradeListDBVar.char.PrimarySpecList.Main.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() };
				elseif priorityfieldnum <= UpgradeListDBVar.char.PrimarySpecList.Main.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.PrimarySpecList.Main.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() });
					for i = UpgradeListDBVar.char.PrimarySpecList.Main.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.PrimarySpecList.Main.Row[i][1] = i;
					end
				end
				UpgradeListDBVar.char.PrimarySpecList.Main.Count = UpgradeListDBVar.char.PrimarySpecList.Main.Count + 1;
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText("Successfully added " .. itemLink .. " to your Primary Spec's Main tab.");
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13) and (UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
				ItemEntryBox:SetText("");
				ItemEntryBox:ClearFocus();
				NoteEntryBox:SetText("");
				NoteEntryBox:ClearFocus();
				priorityfield:SetText("");
				priorityfield:ClearFocus();
				UIDropDownMenu_SetText(DropDown_Button, "Location:");
			elseif OffHandCheck:GetChecked() then --If the user selects the Off Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.PrimarySpecList.Off.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.PrimarySpecList.Off.Row[UpgradeListDBVar.char.PrimarySpecList.Off.Count] = { UpgradeListDBVar.char.PrimarySpecList.Off.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() };
				elseif priorityfieldnum <= UpgradeListDBVar.char.PrimarySpecList.Off.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.PrimarySpecList.Off.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() });
					for i = UpgradeListDBVar.char.PrimarySpecList.Off.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.PrimarySpecList.Off.Row[i][1] = i;
					end
				end
				UpgradeListDBVar.char.PrimarySpecList.Off.Count = UpgradeListDBVar.char.PrimarySpecList.Off.Count + 1;
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText("Successfully added " .. itemLink .. " to your Primary Spec's Off tab.");
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14) and (UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
				ItemEntryBox:SetText("");
				ItemEntryBox:ClearFocus();
				NoteEntryBox:SetText("");
				NoteEntryBox:ClearFocus();
				priorityfield:SetText("");
				priorityfield:ClearFocus();
				UIDropDownMenu_SetText(DropDown_Button, "Location:");
			else
				--Can't tell if the user selected main hand or off hand checkbox
			end
		else --Adding to Secondary Spec List
			if MainHandCheck:GetChecked() then --If the user selects the Main Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.SecondarySpecList.Main.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.SecondarySpecList.Main.Row[UpgradeListDBVar.char.SecondarySpecList.Main.Count] = { UpgradeListDBVar.char.SecondarySpecList.Main.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() };
				elseif priorityfieldnum <= UpgradeListDBVar.char.SecondarySpecList.Main.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.SecondarySpecList.Main.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() });
					for i = UpgradeListDBVar.char.SecondarySpecList.Main.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.SecondarySpecList.Main.Row[i][1] = i;
					end
				end
				UpgradeListDBVar.char.SecondarySpecList.Main.Count = UpgradeListDBVar.char.SecondarySpecList.Main.Count + 1;
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText("Successfully added " .. itemLink .. " to your Secondary Spec's Main tab.");
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13) and (UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
				ItemEntryBox:SetText("");
				ItemEntryBox:ClearFocus();
				NoteEntryBox:SetText("");
				NoteEntryBox:ClearFocus();
				priorityfield:SetText("");
				priorityfield:ClearFocus();
				UIDropDownMenu_SetText(DropDown_Button, "Location:");
			elseif OffHandCheck:GetChecked() then --If the user selects the Off Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.SecondarySpecList.Off.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.SecondarySpecList.Off.Row[UpgradeListDBVar.char.SecondarySpecList.Off.Count] = { UpgradeListDBVar.char.SecondarySpecList.Off.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() };
				elseif priorityfieldnum <= UpgradeListDBVar.char.SecondarySpecList.Off.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.SecondarySpecList.Off.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() });
					for i = UpgradeListDBVar.char.SecondarySpecList.Off.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.SecondarySpecList.Off.Row[i][1] = i;
					end
				end
				UpgradeListDBVar.char.SecondarySpecList.Off.Count = UpgradeListDBVar.char.SecondarySpecList.Off.Count + 1;
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText("Successfully added " .. itemLink .. " to your Secondary Spec's Off tab.");
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14) and (UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
				ItemEntryBox:SetText("");
				ItemEntryBox:ClearFocus();
				NoteEntryBox:SetText("");
				NoteEntryBox:ClearFocus();
				priorityfield:SetText("");
				priorityfield:ClearFocus();
				UIDropDownMenu_SetText(DropDown_Button, "Location:");
			else
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText("ERROR: Unable to determine if you selected Main Hand or Off Hand for this weapon");
			end
		end
		WeaponInput:Hide();
		OffHandCheck:SetChecked(false);
		MainHandCheck:SetChecked(true);
		UpgradeListMainFrame:EnableMouse(true)
		CloseButton:EnableMouse(true)
		ToggleButton:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
		OptionsFrame:EnableMouse(true)
		ItemEntryBox:EnableMouse(true)
		DropDown_Button:EnableMouse(true)
		NoteEntryBox:EnableMouse(true)
		PrimarySpecRadioButton:EnableMouse(true)
		SecondarySpecRadioButton:EnableMouse(true)
		priorityfield:EnableMouse(true)
		AddButton:EnableMouse(true)
		tofield:EnableMouse(true)
		fromfield:EnableMouse(true)
		MoveButton:EnableMouse(true)
		DeleteButton:EnableMouse(true)
		deletefield:EnableMouse(true)
	end);
	
	local WeaponInputCancel = CreateFrame('Button', "WeaponInputCancel", WeaponInput, "UIPanelButtonTemplate");
	WeaponInputCancel:SetPoint("LEFT", WeaponInputOK, "RIGHT", 0, 0);
	WeaponInputCancel:SetWidth(100);
	WeaponInputCancel:SetHeight(24);
	WeaponInputCancel:SetText("Cancel");
	WeaponInputCancel:SetScript("OnClick", function (self, button, down)
		WeaponInput:Hide();
		OffHandCheck:SetChecked(false);
		MainHandCheck:SetChecked(true);
		UpgradeListMainFrame:EnableMouse(true)
		CloseButton:EnableMouse(true)
		ToggleButton:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
		OptionsFrame:EnableMouse(true)
		ItemEntryBox:EnableMouse(true)
		DropDown_Button:EnableMouse(true)
		NoteEntryBox:EnableMouse(true)
		PrimarySpecRadioButton:EnableMouse(true)
		SecondarySpecRadioButton:EnableMouse(true)
		priorityfield:EnableMouse(true)
		AddButton:EnableMouse(true)
		tofield:EnableMouse(true)
		fromfield:EnableMouse(true)
		MoveButton:EnableMouse(true)
		DeleteButton:EnableMouse(true)
		deletefield:EnableMouse(true)
	end);
	
	--Frame for asking whether an acquired upgrade is for Main Spec or Off Spec
	local SpecInput = CreateFrame("Frame", "SpecInput", UIParent);
	SpecInput:SetFrameStrata("FULLSCREEN");
	SpecInput:SetWidth(250);
	SpecInput:SetHeight(200);
	
	SpecInput:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	SpecInput:SetBackdropColor(0,0,0,1);
	SpecInput:SetPoint("CENTER", UIParent, "CENTER");
	
	SpecInput:Hide();
	
	local SpecInputHeading = CreateFrame('Frame', "SpecInputHeading", SpecInput);
	SpecInputFont = SpecInputHeading:CreateFontString("SpecInputHeading", "ARTWORK", "GameFontNormal");
	SpecInputFont:SetText("Is your newly acquired item for your Main Spec or your Off Spec?");
	SpecInputFont:SetJustifyH("CENTER");
	SpecInputFont:SetPoint("TOP", SpecInput, "TOP", 0, -15);
	SpecInputFont:SetWidth(230);
	SpecInputFont:SetHeight(SpecInputFont:GetStringHeight());
	
	--Main Spec selected
	local MainSpecCheck = CreateFrame('CheckButton', "MainSpecCheck", SpecInput, "UICheckButtonTemplate");
	MainSpecCheck:SetPoint("CENTER", SpecInput, "CENTER", -30, 25);
	MainSpecCheck:SetChecked(true);
	_G[MainSpecCheck:GetName() .. "Text"]:SetText("Main Spec");
	MainSpecCheck:SetScript("OnClick", function (self, button, down)
		OffSpecCheck:SetChecked(false);
		MainSpecCheck:SetChecked(true);
	end);
	
	--Off Spec selected
	local OffSpecCheck = CreateFrame('CheckButton', "OffSpecCheck", SpecInput, "UICheckButtonTemplate");
	OffSpecCheck:SetPoint("TOP", MainHandCheck, "BOTTOM", 0, 0);
	OffSpecCheck:SetChecked(false);
	_G[OffSpecCheck:GetName() .. "Text"]:SetText("Off Spec");
	OffSpecCheck:SetScript("OnClick", function (self, button, down)
		OffSpecCheck:SetChecked(true);
		MainSpecCheck:SetChecked(false);
	end);
	
	local SpecInputOK = CreateFrame('Button', "SpecInputOK", SpecInput, "UIPanelButtonTemplate");
	SpecInputOK:SetPoint("BOTTOM", SpecInput, "BOTTOM", -50, 20);
	SpecInputOK:SetWidth(100);
	SpecInputOK:SetHeight(24);
	SpecInputOK:SetText("Ok");
		
	local SpecInputCancel = CreateFrame('Button', "SpecInputCancel", SpecInput, "UIPanelButtonTemplate");
	SpecInputCancel:SetPoint("LEFT", SpecInputOK, "RIGHT", 0, 0);
	SpecInputCancel:SetWidth(100);
	SpecInputCancel:SetHeight(24);
	SpecInputCancel:SetText("Cancel");
	SpecInputCancel:SetScript("OnClick", function (self, button, down)
		SpecInput:Hide();
		OffSpecCheck:SetChecked(false);
		MainSpecCheck:SetChecked(true);
		UpgradeListMainFrame:EnableMouse(true)
		CloseButton:EnableMouse(true)
		ToggleButton:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
		OptionsFrame:EnableMouse(true)
		ItemEntryBox:EnableMouse(true)
		DropDown_Button:EnableMouse(true)
		NoteEntryBox:EnableMouse(true)
		PrimarySpecRadioButton:EnableMouse(true)
		SecondarySpecRadioButton:EnableMouse(true)
		priorityfield:EnableMouse(true)
		AddButton:EnableMouse(true)
		tofield:EnableMouse(true)
		fromfield:EnableMouse(true)
		MoveButton:EnableMouse(true)
		DeleteButton:EnableMouse(true)
		deletefield:EnableMouse(true)
	end);
	
	--Frame for asking whether an acquired upgrade is for Main Hand or Off Hand
	local MainHandorOffHand = CreateFrame("Frame", "MainHandorOffHand", UIParent);
	MainHandorOffHand:SetFrameStrata("FULLSCREEN");
	MainHandorOffHand:SetWidth(250);
	MainHandorOffHand:SetHeight(200);
	
	MainHandorOffHand:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	MainHandorOffHand:SetBackdropColor(0,0,0,1);
	MainHandorOffHand:SetPoint("CENTER", UIParent, "CENTER");
	
	MainHandorOffHand:Hide();
	
	local MainHandorOffHandHeading = CreateFrame('Frame', "MainHandorOffHandHeading", MainHandorOffHand);
	MainHandorOffHandFont = MainHandorOffHandHeading:CreateFontString("MainHandorOffHandHeading", "ARTWORK", "GameFontNormal");
	MainHandorOffHandFont:SetText("Is your newly acquired item for your Main Hand or your Off Hand?");
	MainHandorOffHandFont:SetJustifyH("CENTER");
	MainHandorOffHandFont:SetPoint("TOP", MainHandorOffHand, "TOP", 0, -15);
	MainHandorOffHandFont:SetWidth(230);
	MainHandorOffHandFont:SetHeight(MainHandorOffHandFont:GetStringHeight());
	
	--Main Hand weapon check
	local WeaponInMain = CreateFrame('CheckButton', "WeaponInMain", MainHandorOffHand, "UICheckButtonTemplate");
	WeaponInMain:SetPoint("CENTER", MainHandorOffHand, "CENTER", -30, 25);
	WeaponInMain:SetChecked(true);
	_G[WeaponInMain:GetName() .. "Text"]:SetText("Main Hand");
	WeaponInMain:SetScript("OnClick", function (self, button, down)
		WeaponInOff:SetChecked(false);
		WeaponInMain:SetChecked(true);
	end);
	
	--Off Hand weapon check
	local WeaponInOff = CreateFrame('CheckButton', "WeaponInOff", MainHandorOffHand, "UICheckButtonTemplate");
	WeaponInOff:SetPoint("TOP", MainHandCheck, "BOTTOM", 0, 0);
	WeaponInOff:SetChecked(false);
	_G[WeaponInOff:GetName() .. "Text"]:SetText("Off Hand");
	WeaponInOff:SetScript("OnClick", function (self, button, down)
		WeaponInOff:SetChecked(true);
		WeaponInMain:SetChecked(false);
	end);
	
	local MainHandorOffHandOK = CreateFrame('Button', "MainHandorOffHandOK", MainHandorOffHand, "UIPanelButtonTemplate");
	MainHandorOffHandOK:SetPoint("BOTTOM", MainHandorOffHand, "BOTTOM", -50, 20);
	MainHandorOffHandOK:SetWidth(100);
	MainHandorOffHandOK:SetHeight(24);
	MainHandorOffHandOK:SetText("Ok");
	
	local MainHandorOffHandCancel = CreateFrame('Button', "MainHandorOffHandCancel", MainHandorOffHand, "UIPanelButtonTemplate");
	MainHandorOffHandCancel:SetPoint("LEFT", MainHandorOffHandOK, "RIGHT", 0, 0);
	MainHandorOffHandCancel:SetWidth(100);
	MainHandorOffHandCancel:SetHeight(24);
	MainHandorOffHandCancel:SetText("Cancel");
	MainHandorOffHandCancel:SetScript("OnClick", function (self, button, down)
		MainHandorOffHand:Hide();
		WeaponInOff:SetChecked(false);
		WeaponInMain:SetChecked(true);
		UpgradeListMainFrame:EnableMouse(true)
		CloseButton:EnableMouse(true)
		ToggleButton:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
		OptionsFrame:EnableMouse(true)
		ItemEntryBox:EnableMouse(true)
		DropDown_Button:EnableMouse(true)
		NoteEntryBox:EnableMouse(true)
		PrimarySpecRadioButton:EnableMouse(true)
		SecondarySpecRadioButton:EnableMouse(true)
		priorityfield:EnableMouse(true)
		AddButton:EnableMouse(true)
		tofield:EnableMouse(true)
		fromfield:EnableMouse(true)
		MoveButton:EnableMouse(true)
		DeleteButton:EnableMouse(true)
		deletefield:EnableMouse(true)
	end);
	
	--Frame for entering a new note
	local EditNote = CreateFrame("Frame", "EditNote", UIParent);
	EditNote:SetFrameStrata("FULLSCREEN");
	EditNote:SetWidth(360);
	EditNote:SetHeight(150);
	
	EditNote:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
	edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
	tile = true, tileSize = 32, edgeSize = 32, 
	insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	EditNote:SetBackdropColor(0,0,0,1);
	EditNote:SetPoint("CENTER", UIParent, "CENTER");
	
	EditNote:Hide();
	
	local EditNoteHeading = CreateFrame('Frame', "EditNoteHeading", EditNote);
	EditNoteFont = EditNoteHeading:CreateFontString("EditNoteHeading", "ARTWORK", "GameFontNormal");
	EditNoteFont:SetText("Enter your new note:");
	EditNoteFont:SetJustifyH("CENTER");
	EditNoteFont:SetPoint("TOP", EditNote, "TOP", 0, -15);
	EditNoteFont:SetWidth(230);
	EditNoteFont:SetHeight(EditNoteFont:GetStringHeight());
	
	local EditNoteBox = CreateFrame("Editbox", "EditNoteBox", EditNote, "InputBoxTemplate");
	EditNoteBox:SetPoint("CENTER", EditNote, "CENTER", 0, 0);
	EditNoteBox:SetAutoFocus(true);
	EditNoteBox:SetWidth(285);
	EditNoteBox:SetHeight(27);
	EditNoteBox:SetMaxLetters(28);
	
	local EditNoteOK = CreateFrame('Button', "EditNoteOK", EditNote, "UIPanelButtonTemplate");
	EditNoteOK:SetPoint("BOTTOM", EditNote, "BOTTOM", -50, 20);
	EditNoteOK:SetWidth(100);
	EditNoteOK:SetHeight(24);
	EditNoteOK:SetText("Ok");
	
	local EditNoteCancel = CreateFrame('Button', "EditNoteCancel", EditNote, "UIPanelButtonTemplate");
	EditNoteCancel:SetPoint("LEFT", EditNoteOK, "RIGHT", 0, 0);
	EditNoteCancel:SetWidth(100);
	EditNoteCancel:SetHeight(24);
	EditNoteCancel:SetText("Cancel");
	EditNoteCancel:SetScript("OnClick", function (self, button, down)
		EditNote:Hide();
		UpgradeListMainFrame:EnableMouse(true)
		CloseButton:EnableMouse(true)
		ToggleButton:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
		OptionsFrame:EnableMouse(true)
		ItemEntryBox:EnableMouse(true)
		DropDown_Button:EnableMouse(true)
		NoteEntryBox:EnableMouse(true)
		PrimarySpecRadioButton:EnableMouse(true)
		SecondarySpecRadioButton:EnableMouse(true)
		priorityfield:EnableMouse(true)
		AddButton:EnableMouse(true)
		tofield:EnableMouse(true)
		fromfield:EnableMouse(true)
		MoveButton:EnableMouse(true)
		DeleteButton:EnableMouse(true)
		deletefield:EnableMouse(true)
		EditNoteBox:SetText("");
	end);
	
	UpgradeListMainFrame:Hide();
end